require(Rcpp)

require(microbenchmark)
library(simplePackage)




writeLines("\nLoading cppSource source file")
time <- proc.time()
sourceCpp("sourceCpp_source.cpp")
loading_time <- proc.time() - time
writeLines("Loading time")
loading_time
R_search <-function(v)
{
    for(number in 0:vector_length)
        {
            number %in% v 
        }
}


R_mean <-function(v)
{
    result <- 0
    for(number in v)
    {
        result + number
    }    
    result = result/length(v)
    result
}

R_matrix_mul <-function(mat_a, mat_b)
{
    result <-matrix(ncol = nrow(mat_a),nrow = ncol(mat_b))
    for(i in 1:nrow(mat_a)){
        for(k in 1:ncol(mat_b)){
            for(j in 1:ncol(mat_a))
                {
                    result[i,j] = result[i,j] + mat_a[i,k] * mat_b[k,j]
                }
        }
    }
}


#generating a matrix



writeLines("\ncompiling inline_search")
compile_time_search <- proc.time()
inline_search <-cppFunction('bool inline_search (std::vector<int> vec)
    {
        bool found = true;

        for (unsigned long i = 0; i < vec.size(); i++) {
            if (std::find(vec.begin(), vec.end(), i) == vec.end()) {
                found = false;
            };
        }
        return found;
    }
')

compile_time_search <- proc.time() - time
writeLines("Compiling time inline search: ")
compile_time_search

writeLines("\ncompiling inline_mean")
compile_time_mean <- proc.time()
inline_mean <-cppFunction('float sCpp_mean(std::vector<int> vec)
    {
        float result = 0;
        for(float entry :  vec)
        {
            result += entry;
        }
        return result/vec.size();
    }
')

compile_time_mean <- proc.time() - time
writeLines("Compiling time inline mean: ")
compile_time_mean

writeLines("\ncompiling inline_matrix_mul")
compile_time_matrix_mul <- proc.time()
inline_matrix_mul <-cppFunction('Rcpp::NumericMatrix sCpp_matrix_mul(Rcpp::NumericMatrix a, Rcpp::NumericMatrix b) {
    unsigned long n = a.nrow();    // hoehe a
    unsigned long m = a.ncol(); // breite a /hoehe b
    unsigned long l = b.ncol(); // breite b
    Rcpp::NumericMatrix result(n, b.ncol());
    if (a.ncol() == b.nrow()) {
        for (unsigned long i = 0; i < n; i++)
            for (unsigned long k = 0; k < l; k++)
                for (unsigned long j = 0; j < m; j++) {
                    result(i,j) += a(i,k) * b(k,j);
                }
    } else {
        std::cout << "Error: matricies dont have the right dimensions " << std::endl;
        exit(EXIT_FAILURE);
    }
    return result;
}')

compile_time_matrix_mul <- proc.time() - time
writeLines("Compiling time inline_matrix_mul: ")
compile_time_matrix_mul

writeLines("\nSummation of inline compile time:")
compile_time_matrix_mul + compile_time_search + compile_time_mean

vector_length   <- 10000
matrix_n        <- 100
matrix_m        <- 100
num_of_repititons <- 100

seq_a <- seq(1:(matrix_n * matrix_m))

mat_a <- matrix(seq_a, matrix_n)

v = rep(0,vector_length)


mean_benchmark <- microbenchmark(
                     R_mean(v),
                     sCpp_mean(v),
                     inline_mean(v),
                     cpp_mean(v),
                     unit="ms", times = num_of_repititons)

matrix_multiplication_benchmark <- microbenchmark( 
                    R_matrix_mul(mat_a, mat_a),
                    sCpp_matrix_mul(mat_a, mat_a),
                    inline_matrix_mul(mat_a, mat_a),
                    cpp_matrix_mul(mat_a, mat_a),
                    unit="ms", times = num_of_repititons)

search_benchmark <- microbenchmark(
                     R_search(v),
                     sCpp_search(v),
                     inline_search(v),
                     cpp_search(v),
                     unit="ms", times = num_of_repititons)


writeLines(" ")
summary(mean_benchmark)
writeLines(" ")
summary(matrix_multiplication_benchmark)
writeLines(" ")
summary(search_benchmark)

